@@ -35,6 +35,7 @@ from cql.marshal import (int8_pack, int8_unpack, uint16_pack, uint16_unpack,
                          varint_pack, varint_unpack)
 from decimal import Decimal
 import time
+import socket
 import calendar
 import re
 
@@ -136,6 +137,8 @@ def lookup_casstype(casstype):
 
     """
 
+    if isinstance(casstype, CassandraType):
+        return casstype
     try:
         return parse_casstype_args(casstype)
     except (ValueError, AssertionError, IndexError), e:
@@ -156,6 +159,8 @@ def lookup_cqltype(cqltype):
 
     """
 
+    if isinstance(cqltype, CassandraType):
+        return cqltype
     args = ()
     if cqltype.startswith("'") and cqltype.endswith("'"):
         return lookup_casstype(cqltype[1:-1].replace("''", "'"))
@@ -406,6 +411,25 @@ class IntegerType(_CassandraType):
     deserialize = staticmethod(varint_unpack)
     serialize = staticmethod(varint_pack)
 
+class InetAddressType(_CassandraType):
+    typename = 'inet'
+
+    @staticmethod
+    def deserialize(byts):
+        if len(byts) == 16:
+            fam = socket.AF_INET6
+        else:
+            fam = socket.AF_INET
+        return socket.inet_ntop(fam, byts)
+
+    @staticmethod
+    def serialize(addr):
+        if ':' in addr:
+            fam = socket.AF_INET6
+        else:
+            fam = socket.AF_INET
+        return socket.inet_pton(fam, addr)
+
 class CounterColumnType(_CassandraType):
     typename = 'counter'
 
@@ -600,8 +624,22 @@ class ColumnToCollectionType(_ParameterizedType):
     information.
     """
     typename = "'org.apache.cassandra.db.marshal.ColumnToCollectionType'"
+    num_subtypes = 'UNKNOWN'
+
+class ReversedType(_ParameterizedType):
+    typename = "'org.apache.cassandra.db.marshal.ReversedType'"
     num_subtypes = 1
 
+    @classmethod
+    def deserialize_safe(cls, byts):
+        subtype, = cls.subtypes
+        return subtype.from_binary(byts)
+
+    @classmethod
+    def serialize_safe(cls, val):
+        subtype, = cls.subtypes
+        return subtype.to_binary(val)
+
 def is_counter_type(t):
     if isinstance(t, basestring):
         t = lookup_casstype(t)
@@ -610,7 +648,7 @@ def is_counter_type(t):
 cql_type_to_apache_class = dict([(c, t.cassname) for (c, t) in _cqltypes.items()])
 apache_class_to_cql_type = dict([(n, t.typename) for (n, t) in _casstypes.items()])
 
-cql_types = sorted(_cqltypes.keys())
+cql_types = sorted([t for t in _cqltypes.keys() if not t.startswith("'")])
 
 def cql_typename(casstypename):
     """
